# frozen_string_literal: true

require 'json'
require_relative 'mtg_api_response'

class Cards
  def self.get(no_cache = nil)
    new(no_cache).get
  end

  def get
    JSON.parse(cards)
  end

  private

  attr_reader :no_cache

  def initialize(no_cache)
    @no_cache = no_cache
  end

  def cards
    fetch_and_save_cards_from_api if no_cache || cache_empty?

    read_cache
  end

  def read_cache
    File.read(cache_file_path)
  end

  def cache_empty?
    File.size(cache_file_path).zero?
  end

  def fetch_and_save_cards_from_api
    File.open(cache_file_path, 'w') do |file|
      file.write(MtgApiResponse.results)
    end
  end

  def cache_file_path
    File.expand_path('../cache/cards.json', __dir__)
  end
end
