# frozen_string_literal: true

require 'typhoeus'
require 'typhoeus/adapters/faraday'
require 'json'

class MtgApiRequest
  URL = 'https://api.magicthegathering.io/v1/cards'
  PARALLEL_COUNT = 40

  attr_reader :results

  def initialize
    @results = []
  end

  def make
    while unfetched_pages.any?
      @temp_results = []
      connection.in_parallel do
        unfetched_pages.sample(PARALLEL_COUNT).each do |page|
          fetch_page(page)
        end
      end

      process_results(@temp_results)
    end
  end

  private

  def unfetched_pages
    @unfetched_pages ||= (1..total_page_count).to_a
  end

  def total_page_count
    response = Faraday.get(URL)
    (response.headers['total-count'].to_f / 100).ceil
  end

  def connection
    @connection ||= Faraday.new(url: URL) { |faraday| faraday.adapter :typhoeus }
  end

  def fetch_page(page)
    url = URL + "?page=#{page}"
    puts "Fetching: #{url}"
    response = connection.get(url)

    @temp_results << [page, response]
  end

  def process_results(temp_results)
    temp_results.each do |page, result|
      next unless result.status == 200

      unfetched_pages.delete(page)
      results << JSON.parse(result.body)
    end
  end
end
